@@ -1,5 +1,5 @@
-import argparse
 import cv2
+import glob
 import itertools
 import multiprocessing
 import networkx
@@ -17,10 +17,13 @@ class Namespace():
         self.__dict__.update(dict)
 
 class Photo():
-    def __init__(self,path):
+    def __init__(self,path,uuid=None):
         self.path = path
         self.uuid = path.split("/")[-1].split(".")[0]
+        if uuid is not None:
+            self.uuid = uuid(self.uuid)
         self.size = os.stat(path).st_size
+        self.original_path = path
 
     def __lt__(self,other):
         if self.size < other.size:
@@ -29,9 +32,15 @@ class Photo():
             return False
         return self.uuid < other.uuid
 
+    def __eq__(self,other):
+        return self.uuid == other.uuid
+
     def __repr__(self):
         return "P(%d,%s)" % (self.size, self.uuid)
 
+    def __hash__(self):
+        return hash(self.uuid)
+
     def represent(self,dimension):
         try:
             img = cv2.imdecode(np.fromfile(self.path,dtype=np.uint8),cv2.IMREAD_UNCHANGED)
@@ -70,21 +79,55 @@ class DuplicateFinder():
         self.cores = cores
         self.max = max_images
 
-    def scan(self,dimension=(50,50),radius=1000,prefix=None):
-        photos = self.load()
-        rep_photos = self.represent(photos,dimension=dimension)
-        classes = self.find(rep_photos,radius=radius)
+    def scan(self,thumbs=False,dimensions=((10,10),(50,50)),radiuses=(200,1000),prefix=None):
+        photos = self.load(thumbs=thumbs)
+        while dimensions and radiuses:
+            dimension, dimensions = dimensions[0], dimensions[1:]
+            radius, radiuses = radiuses[0], radiuses[1:]
+            rep_photos = self.represent(photos,dimension=dimension)
+            classes = self.find(rep_photos,radius=radius)
+            if dimensions and radiuses:
+                photos = list({photo for equiv in classes for photo in equiv})
         if prefix is not None:
             self.tag(classes=classes,prefix=prefix)
         return classes
 
-    def load(self):
-        originals_dir = os.path.join(self.library_dir,"originals")
-        filenames_all = (os.path.join(rootname,filename) for rootname, _, filenames in os.walk(originals_dir) for filename in filenames)
+    def humanize(num, separator=" ", suffix="B"):
+        for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
+            if abs(num) < 1024.0:
+                return f"{num:3.1f}{separator}{unit}{suffix}"
+            num /= 1024.0
+        return f"{num:.1f}Yi{suffix}"
+
+    def _join(self,dirs):
+        return os.path.join(self.library_dir,*dirs)
+
+    def load(self,thumbs=False):
+        images_dir = self._join(["resources","derivatives","masters"]) if thumbs else self._join(["originals"])
+        filenames_all = (os.path.join(rootname,filename) for rootname, _, filenames in os.walk(images_dir) for filename in filenames)
         filenames = (filename for filename in filenames_all if filename.split(".")[-1] == "jpeg")
+        if thumbs:
+            def thumbs_remover(s):
+                return s.split("_")[0]
         d = Description("gathering all photos from",self.library_dir)
         with d:
-            photos = [Photo(filename) for filename in self.tqdm(filenames,desc=d.desc)]
+            photos = [Photo(filename,uuid=thumbs_remover if thumbs else None) for filename in self.tqdm(filenames,desc=d.desc)]
+        if thumbs:
+            d = Description("scanning for originals")
+            with d:
+                originals = (os.path.join(rootname,filename) for rootname, _, filenames in os.walk(self._join(["originals"])) for filename in filenames)
+                originals = {path.split("/")[-1].split(".")[0]:path for path in originals}
+                print(originals)
+            d = Description("checking whether original exists")
+            new_photos = []
+            for photo in self.tqdm(photos,desc=d.desc):
+                original_path = originals.get(photo.uuid,None)
+                if original_path is not None:
+                    photo.original_path = original_path
+                    new_photos.append(photo)
+            photos = new_photos
+            print("INFO: reduced number of images to",len(photos),file=sys.stderr,flush=True)
+        print("INFO: total amount of data to scan is",DuplicateFinder.humanize(sum((photo.size for photo in photos))),file=sys.stderr,flush=True)
         return photos
 
     def _represent(photo,dimension):
